import {getTemplate} from "../../libs/template.js";
import {formatBytes, formatDate, formatDirectoryItemCount, getEntries, openFile} from "../files.js";
import {getCurrentDir, navigate} from "../../libs/navigation.js";
import {getOneElementOrThrow} from "../../libs/element.js";
import {sep} from "../../libs/path.js";

export const renderListFiles = () => {
    const fileEl = getOneElementOrThrow(document, '[data-ui=files]');
    const containerTemplate = getTemplate('files-list');

    const body = containerTemplate.el('body');

    /**
     * @callback ValueFormat
     * @param {any} value
     * @param {Entry} entry
     * @return any
     */

    /**
     * @param {Entry} entry
     * @param {Template} parentEl
     * @param {string} name
     * @param {ValueFormat} valueFormat
     */
    const applyValue = (entry, parentEl, name, valueFormat = (v, e) => v) => {
        if (name in entry) {
            const targetEl = parentEl.el(name);
            targetEl.innerText = valueFormat(entry[name], entry);
        }
    }

    const renderList = async () => {
        const currentDirectory = getCurrentDir();
        const entries = await getEntries(currentDirectory);

        for (const entry of entries) {
            const itemTemplate = getTemplate('files-list-item');

            applyValue(entry, itemTemplate, 'name', (value, entry) => {
                let prefix = '';
                if (entry.is_directory) prefix = sep;
                return `${prefix}${value}`;
            });
            applyValue(entry, itemTemplate, 'modified', (value) => formatDate(value))
            applyValue(
                entry,
                itemTemplate,
                'size',
                (value, entry) => {
                    if (entry.is_directory) {
                        return formatDirectoryItemCount(entry.directory_item_count)
                    } else {
                        return formatBytes(value);
                    }
                }
            )

            const actionEl = itemTemplate.el('action');
            actionEl.addEventListener('click', ({currentTarget}) => {
                fileEl.querySelectorAll('[data-el=body] .bg-blue-500')
                    .forEach(el => el.classList.remove('bg-blue-500'))

                currentTarget.classList.add('bg-blue-500');
            });

            actionEl.addEventListener('dblclick', () => {
                if (entry.is_directory) {
                    navigate(entry.path);
                } else {
                    openFile(entry.path);
                }
            })

            body.appendChild(itemTemplate.render());
        }

        fileEl.innerHTML = '';
        fileEl.appendChild(containerTemplate.render());
    }

    renderList();
}